@@ -2,6 +2,7 @@ from .chisquare import ChiSquareDrift
 from .classifier import ClassifierDrift
 from .ks import KSDrift
 from .mmd import MMDDrift
+from .model_uncertainty import ClassifierUncertaintyDrift, RegressorUncertaintyDrift
 from .tabular import TabularDrift
 
 __all__ = [
@@ -9,5 +10,7 @@ __all__ = [
     "ClassifierDrift",
     "KSDrift",
     "MMDDrift",
-    "TabularDrift"
+    "TabularDrift",
+    "ClassifierUncertaintyDrift",
+    "RegressorUncertaintyDrift"
 ]
